/*******************************************************************************
*                   Copyright 2002, GALILEO TECHNOLOGY, LTD.                   *
* THIS CODE CONTAINS CONFIDENTIAL INFORMATION OF MARVELL.                      *
* NO RIGHTS ARE GRANTED HEREIN UNDER ANY PATENT, MASK WORK RIGHT OR COPYRIGHT  *
* OF MARVELL OR ANY THIRD PARTY. MARVELL RESERVES THE RIGHT AT ITS SOLE        *
* DISCRETION TO REQUEST THAT THIS CODE BE IMMEDIATELY RETURNED TO MARVELL.     *
* THIS CODE IS PROVIDED "AS IS". MARVELL MAKES NO WARRANTIES, EXPRESSED,       *
* IMPLIED OR OTHERWISE, REGARDING ITS ACCURACY, COMPLETENESS OR PERFORMANCE.   *
*                                                                              *
* MARVELL COMPRISES MARVELL TECHNOLOGY GROUP LTD. (MTGL) AND ITS SUBSIDIARIES, *
* MARVELL INTERNATIONAL LTD. (MIL), MARVELL TECHNOLOGY, INC. (MTI), MARVELL    *
* SEMICONDUCTOR, INC. (MSI), MARVELL ASIA PTE LTD. (MAPL), MARVELL JAPAN K.K.  *
* (MJKK), GALILEO TECHNOLOGY LTD. (GTL) AND GALILEO TECHNOLOGY, INC. (GTI).    *
********************************************************************************
* gtEther.c  
*
* DESCRIPTION:                                                                
*       Ethernet related functions.
*
* DEPENDENCIES:
*
*******************************************************************************/

/* includes */
#include "gtEther.h"
#include "EagleReg.h"
                              
#define EAGLE_MAC_READ(x,y) *y = *(volatile unsigned int *) (0x80008000 + x)
#define EAGLE_MAC_WRITE(x,y) *(volatile unsigned int *) (0x80008000 + x) = y
#define EAGLE_MAC_READ_IMM(x) * (volatile unsigned int *) (0x80008000 + x)

/*******************************************************************************
* gtEtherStartTransmit - start transmit from the given port 
*
* DESCRIPTION:
*       Write first tx desc address in current tx descriptor register for the
*       given priority queue, and start transmitting through this queue.
*
* INPUT:
*       port - port to transmit from                                       
*       firstTxDescAddr - address of first desc                            
*       prio - priority queue to transmit from  
*
* OUTPUT:
*       None.
*
* RETURN:
*       None.
*
*******************************************************************************/
void gtEtherStartTransmit(ETH_PORT port, unsigned int firstTxDescAddr, 
                          ETH_TX_PRIO prio)
{
  unsigned int buff, i;
    /* write the first tx desc address to  CDP register */
    EAGLE_MAC_WRITE(ETH_CURRENT_TX_DESCRIPTOR_POINTER(port, prio),
                                               VIRTUAL_TO_PHY(firstTxDescAddr));
    buff = * (unsigned int *) ((unsigned int)firstTxDescAddr + 8);
    /*    armond_printf ("buffer at %08x\n",buff);
    for (i = 0 ; i < 60 ; i++) {
//      armond_printf ("%x : %02x\n",buff + i, * (unsigned char *) (buff + i));
      }*/
    /* start tx command */
    gtEtherStartTx(port, prio);
}

/*******************************************************************************
* gtEtherEnableReception - enables rx queue for reception 
*
* DESCRIPTION:
*       Write first rx desc address in current rx descriptor register and
*       first rx desc register for the given priority queue, and enables
*       reception through this queue.
*
* INPUT:
*       port - port to transmit from                                       
*       firstRxDescAddr - address of first desc                            
*       prio - priority queue to transmit from  
*
* OUTPUT:
*       None.
*
* RETURN:
*       None.
*
*******************************************************************************/
void gtEtherEnableReception(ETH_PORT port, unsigned int firstRxDescAddr, 
                            ETH_RX_PRIO prio)
{
    /* write the first rx desc address to  FDP and CDP registers */
    EAGLE_MAC_WRITE(ETH_CURRENT_RX_DESCRIPTOR_POINTER(port, prio),
                                               VIRTUAL_TO_PHY(firstRxDescAddr));
    EAGLE_MAC_WRITE(ETH_FIRST_RX_DESCRIPTOR_POINTER(port, prio),
                                               VIRTUAL_TO_PHY(firstRxDescAddr));
    /* enable rx command */
    EAGLE_MAC_WRITE(ETH_SDMA_COMMAND_REG(port), ETH_ENABLE_RX); 
}


/*******************************************************************************
* gtEtherStartTx - set start tx bit
*
* DESCRIPTION:
*       Set start tx bit for a given port and priority queue. User should 
*       initialize current tx descriptor register before calling this function. 
*
* INPUT:
*       port - port to transmit from                                       
*       prio - priority queue to transmit from  
*
* OUTPUT:
*       None.
*
* RETURN:
*       None.
*
*******************************************************************************/
void gtEtherStartTx(ETH_PORT port, ETH_TX_PRIO prio)
{
    /* start tx command */
    GT_SET_REG_BITS(0x80008000 + ETH_SDMA_COMMAND_REG(port), ETH_START_TX_PRIO0 >> prio); 
}

/*******************************************************************************
* gtEtherEnableRx - enables rx quere for reception 
*
* DESCRIPTION:
*       Set enable rx bit for a given port and priority queue. User should 
*       initialize current and first rx descriptor registers before calling this 
*       function. 
*
* INPUT:
*       port - port to transmit from                                       
*       prio - priority queue to transmit from  
*
* OUTPUT:
*       None.
*
* RETURN:
*       None.
*
*******************************************************************************/
void gtEtherEnableRx(ETH_PORT port, ETH_RX_PRIO prio)
{
    /* enable rx command */
    GT_SET_REG_BITS(0x80008000 + ETH_SDMA_COMMAND_REG(port), ETH_ENABLE_RX); 
}

/*******************************************************************************
* gtEtherStopTx - stop transmission 
*   
* DESCRIPTION:
*       Stops transmission in a given priority queue of a given port
*
* INPUT:
*       port - the transmitting port
*       prio - pririty of transmitting queue to be stopped
*
* OUTPUT:
*       None.
*
* RETURN:
*       None.
*
*******************************************************************************/
void gtEtherStopTx(ETH_PORT port, ETH_TX_PRIO prio)
{
    int prioBit;
    if(prio == ETH_TX_PRIO0)
    {
        prioBit = ETH_STOP_TRANSMIT_PRIO0;
    }
    else prioBit = ETH_STOP_TRANSMIT_PRIO1;
    GT_SET_REG_BITS(0x80008000 + ETH_SDMA_COMMAND_REG(port), prioBit); 
}

/*******************************************************************************
* gtEtherAbortTx - abort transmission 
*   
* DESCRIPTION:
*       Aborts transmission in a given port.
*
* INPUT:
*       port - the transmitting port.
*      
* OUTPUT:
*       None.
*
* RETURN:
*       None.
*
*******************************************************************************/
void gtEtherAbortTx(ETH_PORT port)
{
    GT_SET_REG_BITS(0x80008000 + ETH_SDMA_COMMAND_REG(port), ETH_ABORT_TX); 
}

/*******************************************************************************
* gtEtherIsAbortTxEnded - Checks if Tx abort sequence has finished.
*   
* DESCRIPTION:
*       Checks if ABORT_TX bit is reset (after being set for abort command).
*
* INPUT:
*       port - the transmitting port 
*
* OUTPUT:
*       None.
*
* RETURN:
*       true - abort sequence has finished, ABORT_TX bit is reset.
*       false - ABORT_TX bit is still set
*
*******************************************************************************/
bool gtEtherIsAbortTxEnded(ETH_PORT port)
{
    if(EAGLE_MAC_READ_IMM(ETH_SDMA_COMMAND_REG(port)) & ETH_ABORT_TX)
    {
        return false;
    }
    return true;
}

/*******************************************************************************
* gtEtherAbortRx - Abort reception.
*   
* DESCRIPTION:
*       Aborts reception in a given port.
*
* INPUT:
*       port - the receiving port
*      
* OUTPUT:
*       None.
*
* RETURN:
*       None.
*
*******************************************************************************/
void gtEtherAbortRx(ETH_PORT port)
{
    GT_SET_REG_BITS(0x80008000 + ETH_SDMA_COMMAND_REG(port), ETH_ABORT_RX); 
}

/*******************************************************************************
* gtEtherIsAbortRxEnded - Checks if Rx abort sequence has finished.
*   
* DESCRIPTION:
*       Checks if ABORT_RX bit is reset (after being set for abort command).
*
* INPUT:
*       port - the receiving port 
*
* OUTPUT:
*       None.
*
* RETURN:
*       true - abort sequence has finished, ABORT_RX bit is reset.
*       false - ABORT_RX bit is still set
*
*******************************************************************************/
bool gtEtherIsAbortRxEnded(ETH_PORT port)
{
    if(EAGLE_MAC_READ_IMM(ETH_SDMA_COMMAND_REG(port)) & ETH_ABORT_RX)
    {
        return false;
    }
    return true;
}

/*******************************************************************************
* gtEtherInitHashTable - initialize the hash table
*
* DESCRIPTION:
*        This function initializes the hash table of the given eth port,  
*        according to inputs from the user: sets the size and mode in the port's
*        configuration register, and clears memory area for table from
*        hashTableBase.
*
* INPUT:
*         port - the eth port
*         mode - hash function mode
*         size - hash table size
*         hashTableBase - base address of hash table
*         numOfEntries - num of entries in entries array (can be 0)
*         entries - array of entries to insert in table. If no entries are 
*                   entered in initialization put NULL and put numOfEntries = 0. 
*                   If numOfEntries>0 the function calls gtEtherAddAddress() for
*                   every entry.
*
* OUTPUT:
*        None.
*
* RETURN: 
*         HASH_TABLE_SUCCESS - If initialization succeeded
*         HASH_TABLE_BASE_NOT_ALIGNED - If hash base address is not aligned to
*                                       0x1000    
*         UNABLE_TO_ADD_ADDRESS - If unable to add address from entries array
*
*******************************************************************************/
ETH_HASH_TABLE_STATUS gtEtherInitHashTable(ETH_PORT port, ETH_HASH_MODE mode, 
                                           ETH_HASH_TABLE_SIZE size, 
                                           unsigned int hashTableBase, 
                                           unsigned int numOfEntries, 
                                           HASH_TABLE_ENTRY_DATA* entries)
{
    unsigned int i, tableSizeInBytes;

    if((hashTableBase & 0x7) != 0)
    {
        return HASH_TABLE_BASE_NOT_ALIGNED;
    }

    EAGLE_MAC_WRITE(ETH_HASH_TABLE_POINTER_REG(port), 
                                                VIRTUAL_TO_PHY(hashTableBase));
    if(mode == ETH_HASH_MODE_0) 
    {
        GT_RESET_REG_BITS(0x80008000 + ETH_PORT_CONFIG_REG(port), ETH_HASH_FUNCTION_BIT);
    }
    else
    {
        GT_SET_REG_BITS(0x80008000 + ETH_PORT_CONFIG_REG(port), ETH_HASH_FUNCTION_BIT);
    }
//    armond_printf ("ProbAABL.'.. \n");

    if(size == _8K_ADDRESSES_TABLE)
    {
        tableSizeInBytes=256 * _1K;
        GT_RESET_REG_BITS(0x80008000 + ETH_PORT_CONFIG_REG(port), ETH_HASH_SIZE_BIT);
    }
    else
    {
        tableSizeInBytes=16 * _1K;
        GT_SET_REG_BITS(0x80008000 + ETH_PORT_CONFIG_REG(port), ETH_HASH_SIZE_BIT);
    }
   /* clear hash table */
    for(i = 0; i < tableSizeInBytes; i+=4)
    {
        GT_WRITE_WORD(PHY_TO_VIRTUAL(hashTableBase + i), 0);
    }
//     armond_printf ("ProbAABL.'..2 \n");
   for(i = 0; i < numOfEntries; i++)
    {
        if ((gtEtherAddEntryToHash(port, &(entries[i]))) == 
                                                        UNABLE_TO_ADD_ADDRESS)
        {
            return UNABLE_TO_ADD_ADDRESS;
        }
    }
    return HASH_TABLE_SUCCESS;
}

/*******************************************************************************
* gtEtherAddEntryToHash - Adds a new entry to the hash table
*
* DESCRIPTION:
*       The function calculates the entry address to a given mac address, 
*       according to hash size and mode, as set in port configuration register.
*       The function will try ETH_HASH_HOP_NUMBER times to find an empty entry
*       and if fails returns UNABLE_TO_ADD_ADDRESS.
*       
* INPUT:  
*           port.
*           entryData - entry to be add to table with the fields macLow, 
                        macHigh, prio and rdBit. 
*                       priority field and rd bit will be set as defined 
*                       in entryData. Valid bit will be set and skip bit reset.
*
* OUTPUT:
*       None.
*
* RETURN:
*       HASH_TABLE_SUCCESS - if succeeded
*       UNABLE_TO_ADD_ADDRESS - if after ETH_HASH_HOP_NUMBER unable to find an
*                               empty entry for new address
*
*******************************************************************************/
ETH_HASH_TABLE_STATUS gtEtherAddEntryToHash(ETH_PORT port, 
                                              HASH_TABLE_ENTRY_DATA* entryData)
{
    unsigned int     hashTableBase;
    unsigned int     size, tableOffset;
    /* low - bits 31:0, high - 47:32 */                    
    unsigned int     ethAddressLow, ethAddressHigh;  
    unsigned int     ethAddressLowInversed, ethAddressHighInversed;
    /* for hash function calculation */
    unsigned int     ethAdd0,ethAdd1,ethAdd2,ethAdd3; 
    unsigned int     hashResult;  /* only bits 14:0 matter! */
    unsigned int     i;
    unsigned int     entryAddress;
    HASH_TABLE_ENTRY *entry;
    /* calculate entry data (same for any hash mode or size of table) */
    ethAddressLow = ((entryData->macHigh >> 8) & 0x000000ff) |  /* bits 7:0 */
                    ((entryData->macHigh << 8) & 0x0000ff00) |  /* bits 15:8 */
                    ((entryData->macLow >> 8) & 0x00ff0000)  |  /* bits 23:16 */
                    ((entryData->macLow << 8) & 0xff000000);    /* bits 31:24 */
    
    ethAddressHigh = ((entryData->macLow >> 8) & 0x000000ff) |  /* bits 39:32 */
                     ((entryData->macLow << 8) & 0x0000ff00);   /* bits 47:40 */
         
    /* calculate entry address: */
    ethAddressLowInversed = (SWAP_NIBBLES(entryData->macLow) |
                            (SWAP_NIBBLES(entryData->macLow >> 8) << 8) | 
                            (SWAP_NIBBLES(entryData->macLow >> 16) << 16) | 
                            (SWAP_NIBBLES(entryData->macLow >> 24) << 24));

    ethAddressHighInversed = (SWAP_NIBBLES(entryData->macHigh) |
                             (SWAP_NIBBLES(entryData->macHigh >> 8) << 8)); 

    ethAddressLowInversed = (INVERSE_NIBBLE(ethAddressLowInversed) |
                           (INVERSE_NIBBLE(ethAddressLowInversed >> 4) << 4) |  
                           (INVERSE_NIBBLE(ethAddressLowInversed >> 8) << 8) |  
                           (INVERSE_NIBBLE(ethAddressLowInversed >> 12) << 12) |  
                           (INVERSE_NIBBLE(ethAddressLowInversed >> 16) << 16) |  
                           (INVERSE_NIBBLE(ethAddressLowInversed >> 20) << 20) |  
                           (INVERSE_NIBBLE(ethAddressLowInversed >> 24) << 24) |  
                           (INVERSE_NIBBLE(ethAddressLowInversed >> 28) << 28));

   ethAddressHighInversed = (INVERSE_NIBBLE(ethAddressHighInversed) |
                            (INVERSE_NIBBLE(ethAddressHighInversed >> 4) << 4) |  
                            (INVERSE_NIBBLE(ethAddressHighInversed >> 8) << 8) |  
                          (INVERSE_NIBBLE(ethAddressHighInversed >> 12) << 12)); 

    hashTableBase=EAGLE_MAC_READ_IMM(ETH_HASH_TABLE_POINTER_REG(port));
    if((EAGLE_MAC_READ_IMM(ETH_PORT_CONFIG_REG(port))) & ETH_HASH_FUNCTION_1)
    {
        ethAdd0 =  INVERSE_6_BITS(ethAddressLowInversed) << 9;  /* bits 14:9 */
        /* bits 6:14 of ethAdd */
		ethAdd1 =  INVERSE_9_BITS(((ethAddressLowInversed >> 6)));  
        /* bits 15:23 of ethAdd */
		ethAdd2 =  INVERSE_9_BITS((ethAddressLowInversed >> 15));   
        i = (ethAddressLowInversed >> 24) | 
                        (0x100 & (ethAddressHighInversed << 8)); 
		ethAdd3 =  INVERSE_9_BITS(i); /* bits 24:32  of ethAdd */
    }
    else    /* HASH FUNCTION 0 */
    {
        ethAdd0 = (ethAddressLowInversed << 7) & 0x7e00;   /* bits 14:9 */
        ethAdd1 = ((ethAddressLowInversed >> 6) & 0x1fc) | 
                   (ethAddressLowInversed & 0x3); 
        ethAdd2 = (ethAddressLowInversed >> 15) & 0x1ff;
        ethAdd3 = (ethAddressLowInversed >>24) |
                  (0x100 & (ethAddressHighInversed << 8));
    }
    hashResult = ethAdd0 ^ ethAdd1 ^ ethAdd2 ^ ethAdd3;
    if((EAGLE_MAC_READ_IMM(ETH_PORT_CONFIG_REG(port))) & ETH_HASH_SIZE_16KB)
    {
        size = 16*_1K;
        tableOffset = (hashResult << 3) & 0x3ff8;
    }
    else
    {
        size = 256*_1K;
        tableOffset = (hashResult << 3) & 0x3fff8;
    }
    for(i = 0; i < ETH_HASH_HOP_NUMBER; i++)
    {
        if((tableOffset + i*8) >= size)
        {
            /* start from hash table base */
            entryAddress = hashTableBase + tableOffset + i*8 - size;
        }
        else
        {
            entryAddress = hashTableBase + tableOffset + i*8;
        }

        entry = (HASH_TABLE_ENTRY*)PHY_TO_VIRTUAL(entryAddress);
        if((entry->lowWord & ETH_HASH_SKIP_BIT) || 
           (entry->lowWord & ETH_HASH_VALID_BIT)) continue;
        /* skip=0, valid=1: */
        entry->lowWord =  
            (ETH_HASH_VALID_BIT | (ethAddressLow << 3)) & 0xfffffff9; 
        if(entryData->rdBit)  /* set RD bit */
        {
            entry->lowWord = entry->lowWord | ETH_HASH_RD_BIT;
        }
        entry->highWord = ((ethAddressLow >> 29) & 0x7) | 
                           (ethAddressHigh << 3) | (entryData->prio << 19);
        break;
    }
    if (i == ETH_HASH_HOP_NUMBER) 
        return UNABLE_TO_ADD_ADDRESS;
    return HASH_TABLE_SUCCESS;
} 

/*******************************************************************************
* gtEtherRemoveEntryFromHash - removes an address from hash table
*
* DESCRIPTION:
*       The function calculates the entry address to a given mac address, 
*       according to hash size and mode, as set in port configuration register.
*       the function will try ETH_HASH_HOP_NUMBER times to remove the address.
*       If address is found in hash table its fields will be reset to zero and
*       skip bit will be set.
*       
* INPUT:  
*           port.
*           entryData - a pointer to a HASH_TABLE_ENTRY_DATA representing the 
*                       entry to be removed. Only macLow and macHigh fields are 
*                       relevant to this function.
*
* OUTPUT:
*       None.
*
* RETURN:
*       HASH_TABLE_SUCCESS - If succeeded.
*       ADDRESS_NOT_IN_TABLE - If after ETH_HASH_HOP_NUMBER unable to find the
*                              address in table.
*
*******************************************************************************/
ETH_HASH_TABLE_STATUS gtEtherRemoveEntryFromHash(ETH_PORT port, 
                                               HASH_TABLE_ENTRY_DATA* entryData)
{
    unsigned int     hashTableBase;
    unsigned int     size, tableOffset;
    /* low - bits 31:0, high - 47:32 */
    unsigned int     ethAddressLow, ethAddressHigh;  
    unsigned int     ethAddressLowInversed, ethAddressHighInversed;
    /* for hash function calculation */
    unsigned int     ethAdd0,ethAdd1,ethAdd2,ethAdd3; 
    unsigned int     hashResult;  /* only bits 14:0 matter! */
    unsigned int     i;
    unsigned int     entryAddress;
    HASH_TABLE_ENTRY *entry;
    /* calculate entry data (same for any hash mode or size of table): */
    ethAddressLow = ((entryData->macHigh >> 8) & 0x000000ff) |  /* bits 7:0 */
                    ((entryData->macHigh << 8) & 0x0000ff00) |  /* bits 15:8 */
                    ((entryData->macLow >> 8) & 0x00ff0000)  |  /* bits 23:16 */
                    ((entryData->macLow << 8) & 0xff000000);    /* bits 31:24 */
    
    ethAddressHigh = ((entryData->macLow>>8) & 0x000000ff) |    /* bits 39:32 */
                     ((entryData->macLow<<8) & 0x0000ff00);     /* bits 47:40 */
         
    /* calculate entry address: */
    ethAddressLowInversed = (SWAP_NIBBLES(entryData->macLow) |
                            (SWAP_NIBBLES(entryData->macLow >> 8) << 8) | 
                            (SWAP_NIBBLES(entryData->macLow >> 16) << 16) | 
                            (SWAP_NIBBLES(entryData->macLow >> 24) << 24));

    ethAddressHighInversed = (SWAP_NIBBLES(entryData->macHigh) |
                             (SWAP_NIBBLES(entryData->macHigh >> 8) << 8)); 

    ethAddressLowInversed = (INVERSE_NIBBLE(ethAddressLowInversed) |
                           (INVERSE_NIBBLE(ethAddressLowInversed >> 4) << 4) |  
                           (INVERSE_NIBBLE(ethAddressLowInversed >> 8) << 8) |  
                           (INVERSE_NIBBLE(ethAddressLowInversed >> 12) << 12) |  
                           (INVERSE_NIBBLE(ethAddressLowInversed >> 16) << 16) |  
                           (INVERSE_NIBBLE(ethAddressLowInversed >> 20) << 20) |  
                           (INVERSE_NIBBLE(ethAddressLowInversed >> 24) << 24) |  
                           (INVERSE_NIBBLE(ethAddressLowInversed >> 28) << 28));

   ethAddressHighInversed = (INVERSE_NIBBLE(ethAddressHighInversed) |
                            (INVERSE_NIBBLE(ethAddressHighInversed >> 4) << 4) |  
                            (INVERSE_NIBBLE(ethAddressHighInversed >> 8) << 8) |  
                          (INVERSE_NIBBLE(ethAddressHighInversed >> 12) << 12)); 

    hashTableBase=EAGLE_MAC_READ_IMM(ETH_HASH_TABLE_POINTER_REG(port));
    if(EAGLE_MAC_READ_IMM(ETH_PORT_CONFIG_REG(port)) & ETH_HASH_FUNCTION_1)
    {
        ethAdd0 =  INVERSE_6_BITS(ethAddressLowInversed) << 9;  /* bits 14:9 */
        /* bits 6:14 of ethAdd */
        ethAdd1 =  INVERSE_9_BITS(((ethAddressLowInversed >> 6)));  
        /* bits 15:23 of ethAdd */
        ethAdd2 =  INVERSE_9_BITS((ethAddressLowInversed >> 15));  
        i = (ethAddressLowInversed >> 24) | 
            (0x100 & ( ethAddressHighInversed << 8)); 
		ethAdd3 =  INVERSE_9_BITS(i); /* bits 24:32  of ethAdd */
    }
    else    /* HASH FUNCTION 0 */
    {
        ethAdd0 = (ethAddressLowInversed << 7) & 0x7e00;   /* bits 14:9 */
        ethAdd1 = ((ethAddressLowInversed >> 6) & 0x1fc) | 
                   (ethAddressLowInversed & 0x3); 
        ethAdd2 = (ethAddressLowInversed >> 15) & 0x1ff;
        ethAdd3 = (ethAddressLowInversed >> 24) |
                  (0x100 & (ethAddressHighInversed << 8));
    }
    hashResult = ethAdd0 ^ ethAdd1 ^ ethAdd2 ^ ethAdd3;
    if((EAGLE_MAC_READ_IMM(ETH_PORT_CONFIG_REG(port))) & ETH_HASH_SIZE_16KB)
    {
        size = 16*_1K;
        tableOffset = (hashResult << 3) & 0x3ff8;
    }
    else
    {
        size=256*_1K;
        tableOffset = (hashResult << 3) & 0x3fff8;
    }
    for(i = 0; i < ETH_HASH_HOP_NUMBER; i++)
    {
        if((tableOffset + i*8) >= size)
        {
            /* start from hash table base: */
            entryAddress = hashTableBase + tableOffset + i*8 - size;
        }
        else
        {
            entryAddress = hashTableBase + tableOffset + i*8;
        }
        entry = (HASH_TABLE_ENTRY*)PHY_TO_VIRTUAL(entryAddress);
        if(entry->lowWord & ETH_HASH_SKIP_BIT) continue;
        if((entry->lowWord & 0xfffffff8) != (ethAddressLow << 3) ||
           (entry->highWord & 0x7ffff) != (((ethAddressLow >> 29) & 0x7) | 
                                          (ethAddressHigh << 3)))
        {
            return ADDRESS_NOT_IN_TABLE;
        }
        entry->highWord = 0x0;
        entry->lowWord = ETH_HASH_SKIP_BIT;
        break;
    }
    if (i == ETH_HASH_HOP_NUMBER) return ADDRESS_NOT_IN_TABLE;
    return HASH_TABLE_SUCCESS;
}

/*****************************************************************************
* etherReadPHYReg - reads a PHY register
*
* DESCRIPTION: 
*       This function will access the given PHY register through the GT SMI
*       register and will read it's value.
* INPUT:
*       port - port's number.
*       PHYReg - the PHY register offset.
*
* OUTPUT: 
*       value - pointer to unsigned int which will receive the value.
*
* RETURN:
*       true - If succeeded.
*       false -  If failed to make the assignment - SMI busy bit is set.
******************************************************************************/
bool gtEtherReadPHYReg(ETH_PORT port, unsigned int PHYReg, unsigned int* value)
{
    unsigned int phyAddr, regValue;
    int          timeOut = 10, i;

    /* first check that it is not busy */
    regValue = EAGLE_MAC_READ_IMM(ETH_SMI_REG);
    while(regValue & ETH_SMI_IS_BUSY)
    {
        for(i = 0; i < 10000000; i++);
        if(timeOut-- < 0)
        {
            return false;
        }
        regValue = EAGLE_MAC_READ_IMM(ETH_SMI_REG);
    }
    /* not busy */
    regValue = EAGLE_MAC_READ_IMM(ETH_PHY_ADDR_REG);
    switch(port)
    {
        case ETH0:
            phyAddr = regValue & 0x1f;
            break;
        case ETH1:
            phyAddr = (regValue >> 5) & 0x1f;
            break;
#ifdef INCLUDE_ETH_PORT_2
        case ETH2:
            phyAddr = (regValue >> 10) & 0x1f;
            break;
#endif /* INCLUDE_ETH_PORT_2 */
        default:
            return false;
    }
    EAGLE_MAC_WRITE(ETH_SMI_REG, (phyAddr << 16) | (PHYReg << 21) | ETH_SMI_READ);

    timeOut = 10; /* initialize the time out var again */ 
    regValue = EAGLE_MAC_READ_IMM(ETH_SMI_REG);
    while(!(regValue & ETH_SMI_READ_VALID))
    {

        for(i = 0; i < 10000000; i++);
        if(timeOut-- < 0 )
        {
            return false;
        }
        regValue = EAGLE_MAC_READ_IMM(ETH_SMI_REG);
    }
    *value = (regValue & 0xffff);
    return true;

}

/*****************************************************************************
* gtEtherWritePHYReg - writes to a PHY register
*
* DESCRIPTION: 
*       This function will access the given PHY register through the GT SMI
*       register and will write a given value to it.
*
* INPUT:
*       port - one of the 3 possible Ethernet ports (0-2).
*       PHYReg - the PHY register offset.
*       value - the value to be written (only bits 0-15!!! PHY regs are 2 bytes)
*
* OUTPUT: 
*       None.
*
* RETURN:
*       true - If succeeded.
*       false -  If failed to make the assignment - SMI busy bit is set.
******************************************************************************/
bool gtEtherWritePHYReg(ETH_PORT port, unsigned int PHYReg, unsigned int value)
{
    unsigned int phyAddr, regValue; 
    int          timeOut = 10, i;

    /* first check that it is not busy */
    regValue = EAGLE_MAC_READ_IMM(ETH_SMI_REG);
    while(regValue & ETH_SMI_IS_BUSY)
    {
        for(i = 0; i < 10000000; i++);
        if(timeOut-- < 1 )
        {
            return false;
        }
    }

    /* not busy */
    regValue = EAGLE_MAC_READ_IMM(ETH_PHY_ADDR_REG);
    switch(port)
    {
        case ETH0:
            phyAddr = regValue & 0x1f;
            break;
        case ETH1:
            phyAddr = (regValue >> 5) & 0x1f;
            break;
#ifdef INCLUDE_ETH_PORT_2
        case ETH2:
            phyAddr = (regValue >> 10) & 0x1f;
            break;
#endif /* INCLUDE_ETH_PORT_2 */
        default:
            return false;
    }
    EAGLE_MAC_WRITE(ETH_SMI_REG, (phyAddr << 16) | (PHYReg << 21) |
                 ((value) & 0xffff));
    return true;
}

/*******************************************************************************       
* gtEtherSetPhyAddress - Changes PHY address for the given port
*
* DESCRIPTION:
*       Change the PHY device address for a given port (change in PHY address 
*       reg).
* INPUT:
*       port.                         
*       newAddress - max 31 (5 bit field)
*
* OUTPUT:
*       None.
*
* RETURN:
*       None.
*
*******************************************************************************/
void gtEtherSetPhyAddress(ETH_PORT port, unsigned int newAddress)
{
    unsigned int phyAddReg = EAGLE_MAC_READ_IMM(ETH_PHY_ADDR_REG);
    switch(port)
    {
        case ETH0:
            EAGLE_MAC_WRITE(ETH_PHY_ADDR_REG,
                         (phyAddReg & 0xffffffe0) | newAddress);
            break;
        case ETH1:
            EAGLE_MAC_WRITE(ETH_PHY_ADDR_REG, 
                         (phyAddReg & 0xfffffc1f) | (newAddress << 5));
            break;
#ifdef INCLUDE_ETH_PORT_2
        case ETH2:
            EAGLE_MAC_WRITE(ETH_PHY_ADDR_REG, 
                         (phyAddReg & 0xffff83ff) | (newAddress << 10));
#endif /* INCLUDE_ETH_PORT_2 */
    }
    
}

/*******************************************************************************
* gtEtherGetPhyAddress - return phy device address for a given port
*
* DESCRIPTION:
*       Return phy device address for a given port, according to PHY address 
*       register.
*
* INPUT:
*       port.
*
* OUTPUT:
*       None.
*
* RETURN:
*       Given port's phy device address.
*
*******************************************************************************/
unsigned int gtEtherGetPhyAddress(ETH_PORT port)
{
    unsigned int regValue = EAGLE_MAC_READ_IMM(ETH_PHY_ADDR_REG);
    return ((regValue >> (5*port)) & 0x1f);
}

/*******************************************************************************
* gtEtherDefaultInit - initiates ETH ports with default parameters.
*
* DESCRIPTION:
*       Initiates port to work in promiscuous mode, RMII mode, MIB counters
*       work in clear, max frame length=2KB and sdma burst size is 64B.
*
* INPUT:
*       eth port
*
* OUTPUT:
*       None.
*
* RETURN:
*       None.
*******************************************************************************/
void gtEtherDefaultInit(ETH_PORT port)
{
    EAGLE_MAC_WRITE(ETH_PORT_CONFIG_REG(port), ETH_PROMISCUOUS_MODE 
                                                             | ETH_PORT_ENABLE);
    EAGLE_MAC_WRITE(ETH_PORT_CONFIG_EXTEND_REG(port), ETH_RMII_ENABLE 
                                                    | ETH_MAX_FRAME_LENGTH_2KB);
    EAGLE_MAC_WRITE(ETH_SDMA_CONFIG_REG(port), EAGLE_MAC_READ_IMM(ETH_SDMA_CONFIG_REG(port)) 
                                                          | ETH_BURST_SIZE_64B);
}









