/*******************************************************************************
*                   Copyright 2002, GALILEO TECHNOLOGY, LTD.                   *
* THIS CODE CONTAINS CONFIDENTIAL INFORMATION OF MARVELL.                      *
* NO RIGHTS ARE GRANTED HEREIN UNDER ANY PATENT, MASK WORK RIGHT OR COPYRIGHT  *
* OF MARVELL OR ANY THIRD PARTY. MARVELL RESERVES THE RIGHT AT ITS SOLE        *
* DISCRETION TO REQUEST THAT THIS CODE BE IMMEDIATELY RETURNED TO MARVELL.     *
* THIS CODE IS PROVIDED "AS IS". MARVELL MAKES NO WARRANTIES, EXPRESSED,       *
* IMPLIED OR OTHERWISE, REGARDING ITS ACCURACY, COMPLETENESS OR PERFORMANCE.   *
*                                                                              *
* MARVELL COMPRISES MARVELL TECHNOLOGY GROUP LTD. (MTGL) AND ITS SUBSIDIARIES, *
* MARVELL INTERNATIONAL LTD. (MIL), MARVELL TECHNOLOGY, INC. (MTI), MARVELL    *
* SEMICONDUCTOR, INC. (MSI), MARVELL ASIA PTE LTD. (MAPL), MARVELL JAPAN K.K.  *
* (MJKK), GALILEO TECHNOLOGY LTD. (GTL) AND GALILEO TECHNOLOGY, INC. (GTI).    *
********************************************************************************
* gtMpsc.c 
*
* DESCRIPTION:
*       This file contains:
*       1. Functions to initialize mpsc port for tx and rx, according to four
*          differnet protocols: HDLC, BYSINC, Uart, Transparent.
*       2. Baud Rate Generators (BRG's) related functions.
*       3. MPSC abort functions.
*
* DEPENDENCIES:
*       gtSdma.c/.h
*       gtDesc.c/.h
*
*******************************************************************************/

/* includes */
#include "gtMpsc.h"
#include "MvCore.h"
#include "gtSdma.h"
#include "EagleReg.h"

/*******************************************************************************
* gtMpscConfigForBISYNC - configures the MPSC for BISYNC with default values 
*
* DESCRIPTION:
*       Initializes the port to work in default configurations. The rate,
*       the VRC parity and the Bisync mode are determined by user.
*       Default configurations are: 
*       CRC mode - CRC-16, Transmit and receive encoder - NRZ, DPLL uses both
*       rising and falling edges for synchronization, receive sync length = 
*       16bit, channel works in EBCDIC control character.
*       Setting the parity set the control char automaticly at the correct
*       parity value.
*
* INPUT:
*         port - the port to be initiated.
*         parity - the VRC parity to set the MPSC.
*         mode - the Bisync mode to set the MPSC.
*         rxClkSrc - Source of rx Clk. put NO_SRC if port is not functioning as 
*                    rx!!!             
*         txClkSrc - Source of tx Clk. put NO_SRC if port is not functioning as 
*                    tx!!! 
*         rxClkRate - desired clock rate of rx. valid only when ClkSrc is one
*                     of the brg's. Put 0 at any other case. 
*         txClkRate - desired clock rate of tx. valid only when ClkSrc is one
*                     of the brg's. Put 0 at any other case.  
*         tclkFreq - in case rxClkSrc or TxClkSrc are BRG0 or BRG1 send T-clock
*                    frequency in this variable (T-clk will be used as the BRG 
*                    source). If another source for the brg is desired use 
*                    gtMpscBrgInit() to configure the BRG.
*
* OUTPUT:
*       rates:
*       An MPSC_ACTUAL_RATES sturcture with fields:
*       rxClkActualRate - when user uses brg as clock src the function derives 
*       the actual rate from T-CLk. the actual rate is not always equal to the 
*       desired one (see gtMpscBrgInit()).
*       txClkActualRate - like rx clk.
*
* RETURN:
*       None.
*
*******************************************************************************/
void gtMpscConfigForBISYNC(MPSC_PORT port, MPSC_BISYNC_PARITY_MODE parity, 
                           MPSC_BISYNC_MODE mode , MPSC_CLOCK_SOURCE rxClkSrc, 
                           unsigned int rxClkRate, MPSC_CLOCK_SOURCE txClkSrc,
                           unsigned int txClkRate, unsigned int tclkFreq,
                           MPSC_ACTUAL_RATES *rates)
{
    GT_MPSC_MRR_CONFIG(port); 
    GT_MPSC_CLOCK_ROUTING_CONFIG(port, rxClkSrc, txClkSrc);
    if(rxClkSrc == BRG0 || rxClkSrc == BRG1)
    {
        gtMpscBrgEnable(rxClkSrc);
        rates->rxClkActualRate = 
            gtMpscBrgInit(rxClkSrc, GT_TCLK, tclkFreq, rxClkRate);
    }
    if(txClkSrc == BRG0 || txClkSrc == BRG1)
    {
        gtMpscBrgEnable(txClkSrc);
        rates->txClkActualRate = 
            gtMpscBrgInit(txClkSrc, GT_TCLK, tclkFreq, txClkRate);
    }   
	

	GT_MPSC_MMCRL_CONFIG(port, 	MPSC_MODE_BISYNC |
								MPSC_CRC16 |
                                MPSC_ENABLE_TRANSMIT |
								MPSC_ENABLE_RECEIVE);

	GT_MPSC_MMCRH_CONFIG(port, 	MPSC_TRANSMIT_ENCODER_NRZ |
								MPSC_RECEIVE_ENCODER_NRZ |
								MPSC_RECEIVE_DATA_WIDTH_8BIT |
								MPSC_TRANSMIT_PREAMBLE_LENGTH_8B |
								GT_MPSC_TPPT(5) |
								MPSC_RECEIVE_SYNC_LENGTH_16_BIT);

	GT_MPSC_CHR4_CONFIG(port, 
						(GT_MPSC_BISYNC_CONTROL_FILTERING(MPSC_DEFAULT_ALL_1)));


	GT_MPSC_CHR5_CONFIG(port,
	  GT_MPSC_BISYNC_CONTROL_CHAR_LOW(0) | GT_MPSC_BISYNC_CONTROL_CHAR_HIGH(0));

	/* Set mpsc to normal BISYNC or TRANSPARENT or AUTO TRANSPARENT */
	if(BISYNC_NORMAL == mode)
		GT_MPSC_MPCR_CONFIG(port, MPSC_BISYNC_TRAILING_PAD);
	else 
		if(BISYNC_TRANSPARENT == mode)
			GT_MPSC_MPCR_CONFIG(port, MPSC_BISYNC_TRAILING_PAD | 
								MPSC_BISYNC_RX_TRANSPARENT_MODE);
		else
			if (BISYNC_AUTO_TRANSPARENT == mode)
			GT_MPSC_MPCR_CONFIG(port, MPSC_BISYNC_TRAILING_PAD | 
								MPSC_BISYNC_AUTO_TRANSPARENT_MODE);

	
	/* Parity mode none */
	if(PARITY_NONE == parity)
	{
    	GT_MPSC_CHR2_CONFIG(port, 
    				   MPSC_BISYNC_RX_PARITY_EVEN | MPSC_BISYNC_TX_PARITY_EVEN);
    	

    	GT_MPSC_CHR1_CONFIG(port, 
                        MPSC_BISYNC_DISCARD_SYNC |
    	  				MPSC_BISYNC_DISCARD_DLE |
   	  					GT_MPSC_BISYNC_SYNC(MPSC_BISYNC_DEFAULT_SYNC_CHAR) |
    	  				GT_MPSC_BISYNC_DLE(MPSC_BISYNC_DEFAULT_DELIMETER_CHAR));
    		

        GT_MPSC_CHR6_CONFIG(port, 
            GT_MPSC_BISYNC_CONTROL_CHAR_LOW(MPSC_BISYNC_ENTRY_VALID |
                                          MPSC_BISYNC_START_OF_TEXT |
                                          MPSC_BISYNC_GENERATE_INTERRUPT_ON_RX |
    									  MPSC_START_OF_TXT) |
    		GT_MPSC_BISYNC_CONTROL_CHAR_HIGH((MPSC_BISYNC_ENTRY_VALID |
                                          MPSC_BISYNC_START_OF_HEADER |
                                          MPSC_BISYNC_GENERATE_INTERRUPT_ON_RX |
    									  MPSC_START_OF_HNDLR)));

        GT_MPSC_CHR7_CONFIG(port, 
            GT_MPSC_BISYNC_CONTROL_CHAR_LOW(MPSC_BISYNC_ENTRY_VALID |
    		  							  MPSC_BISYNC_MOVE_TO_HUNT |
                                          MPSC_BISYNC_GENERATE_INTERRUPT_ON_RX |
    									  MPSC_BISYNC_BCC_NEXT |
    									  MPSC_END_OF_TXT)|
    		GT_MPSC_BISYNC_CONTROL_CHAR_HIGH(MPSC_BISYNC_ENTRY_VALID |
										  MPSC_BISYNC_MOVE_TO_HUNT |
                                          MPSC_BISYNC_GENERATE_INTERRUPT_ON_RX |
    									  MPSC_BISYNC_BCC_NEXT |
										  MPSC_END_INTERMEDIATE_BLOCK));
		GT_MPSC_CHR8_CONFIG(port, 
			GT_MPSC_BISYNC_CONTROL_CHAR_LOW(MPSC_BISYNC_ENTRY_VALID |             
										  MPSC_BISYNC_MOVE_TO_HUNT |            
										  MPSC_BISYNC_GENERATE_INTERRUPT_ON_RX |
										  MPSC_ENQUIRY)|                     
			GT_MPSC_BISYNC_CONTROL_CHAR_HIGH(MPSC_BISYNC_ENTRY_VALID |             
										  MPSC_BISYNC_MOVE_TO_HUNT |            
										  MPSC_BISYNC_GENERATE_INTERRUPT_ON_RX |
										  MPSC_BISYNC_IGNORE_CHAR_IN_TEXT_MODE |
										  MPSC_NACK));
	}

	/* Parity mode EVEN */
	if(PARITY_EVEN == parity)
	{
    	GT_MPSC_CHR2_CONFIG(port, MPSC_BISYNC_RX_ENABLE_VERTICAL_PARITY_CHECK |
        						   MPSC_BISYNC_TX_ENABLE_VERTICAL_PARITY_CHECK |
        						   MPSC_BISYNC_RX_PARITY_EVEN |
        						   MPSC_BISYNC_TX_PARITY_EVEN);
    	

    	GT_MPSC_CHR1_CONFIG(port, 
                   MPSC_BISYNC_DISCARD_SYNC |
    	  		   MPSC_BISYNC_DISCARD_DLE |
    	  		   GT_MPSC_BISYNC_SYNC(MPSC_BISYNC_DEFAULT_SYNC_CHAR_EVEN) |
    	  		   GT_MPSC_BISYNC_DLE(MPSC_BISYNC_DEFAULT_DELIMETER_CHAR_EVEN));
    		

    	GT_MPSC_CHR6_CONFIG(port, 
            GT_MPSC_BISYNC_CONTROL_CHAR_LOW(MPSC_BISYNC_ENTRY_VALID |
                                          MPSC_BISYNC_START_OF_TEXT |
                                          MPSC_BISYNC_GENERATE_INTERRUPT_ON_RX |
    									  MPSC_START_OF_TXT_EVEN) |
    		GT_MPSC_BISYNC_CONTROL_CHAR_HIGH(MPSC_BISYNC_ENTRY_VALID |
                                          MPSC_BISYNC_START_OF_HEADER |
                                          MPSC_BISYNC_GENERATE_INTERRUPT_ON_RX |
    									  MPSC_START_OF_HNDLR_EVEN));


    	GT_MPSC_CHR7_CONFIG(port, 
            GT_MPSC_BISYNC_CONTROL_CHAR_LOW(MPSC_BISYNC_ENTRY_VALID |
    		  							  MPSC_BISYNC_MOVE_TO_HUNT |
    									  MPSC_BISYNC_BCC_NEXT |
                                          MPSC_BISYNC_GENERATE_INTERRUPT_ON_RX |
    									  MPSC_END_OF_TXT_EVEN) |
    		GT_MPSC_BISYNC_CONTROL_CHAR_HIGH(MPSC_BISYNC_ENTRY_VALID |
										  MPSC_BISYNC_MOVE_TO_HUNT |
                                          MPSC_BISYNC_GENERATE_INTERRUPT_ON_RX |
    									  MPSC_BISYNC_BCC_NEXT |
										  MPSC_END_INTERMEDIATE_BLOCK_EVEN));


		GT_MPSC_CHR8_CONFIG(port, 
			GT_MPSC_BISYNC_CONTROL_CHAR_LOW(MPSC_BISYNC_ENTRY_VALID |             
										  MPSC_BISYNC_MOVE_TO_HUNT |            
										  MPSC_BISYNC_GENERATE_INTERRUPT_ON_RX |
										  MPSC_ENQUIRY_EVEN)|                     
			GT_MPSC_BISYNC_CONTROL_CHAR_HIGH(MPSC_BISYNC_ENTRY_VALID |             
										  MPSC_BISYNC_MOVE_TO_HUNT |            
										  MPSC_BISYNC_GENERATE_INTERRUPT_ON_RX |
										  MPSC_BISYNC_IGNORE_CHAR_IN_TEXT_MODE |
										  MPSC_NACK_EVEN));
	}


	/* Parity mode ODD */
	if(PARITY_ODD == parity)
	{
    	GT_MPSC_CHR2_CONFIG(port, MPSC_BISYNC_RX_ENABLE_VERTICAL_PARITY_CHECK |
    						      MPSC_BISYNC_TX_ENABLE_VERTICAL_PARITY_CHECK |
    						      MPSC_BISYNC_RX_PARITY_ODD |
    						      MPSC_BISYNC_TX_PARITY_ODD);
    	
    	GT_MPSC_CHR1_CONFIG(port, 
                    MPSC_BISYNC_DISCARD_SYNC |
    	  			MPSC_BISYNC_DISCARD_DLE |
    	  			GT_MPSC_BISYNC_SYNC(MPSC_BISYNC_DEFAULT_SYNC_CHAR_ODD) |
    	  			GT_MPSC_BISYNC_DLE(MPSC_BISYNC_DEFAULT_DELIMETER_CHAR_ODD));
    		

    	GT_MPSC_CHR6_CONFIG(port, 
            GT_MPSC_BISYNC_CONTROL_CHAR_LOW(MPSC_BISYNC_ENTRY_VALID |
                                          MPSC_BISYNC_START_OF_TEXT |
                                          MPSC_BISYNC_GENERATE_INTERRUPT_ON_RX |
    									  MPSC_START_OF_TXT_ODD) |
    		GT_MPSC_BISYNC_CONTROL_CHAR_HIGH(MPSC_BISYNC_ENTRY_VALID |
                                          MPSC_BISYNC_START_OF_HEADER |
                                          MPSC_BISYNC_GENERATE_INTERRUPT_ON_RX |
    									  MPSC_START_OF_HNDLR_ODD));


    	GT_MPSC_CHR7_CONFIG(port, 
            GT_MPSC_BISYNC_CONTROL_CHAR_LOW(MPSC_BISYNC_ENTRY_VALID |
    		  							  MPSC_BISYNC_MOVE_TO_HUNT |
    									  MPSC_BISYNC_BCC_NEXT |
                                          MPSC_BISYNC_GENERATE_INTERRUPT_ON_RX |
    									  MPSC_END_OF_TXT_ODD) |
    		GT_MPSC_BISYNC_CONTROL_CHAR_HIGH(MPSC_BISYNC_ENTRY_VALID |
										  MPSC_BISYNC_MOVE_TO_HUNT |
                                          MPSC_BISYNC_GENERATE_INTERRUPT_ON_RX |
    									  MPSC_BISYNC_BCC_NEXT |
										  MPSC_END_INTERMEDIATE_BLOCK_ODD));


		GT_MPSC_CHR8_CONFIG(port, 
			GT_MPSC_BISYNC_CONTROL_CHAR_LOW(MPSC_BISYNC_ENTRY_VALID |             
										  MPSC_BISYNC_MOVE_TO_HUNT |            
										  MPSC_BISYNC_GENERATE_INTERRUPT_ON_RX |
										  MPSC_ENQUIRY_ODD)|                     
			GT_MPSC_BISYNC_CONTROL_CHAR_HIGH(MPSC_BISYNC_ENTRY_VALID |             
										  MPSC_BISYNC_MOVE_TO_HUNT |            
										  MPSC_BISYNC_GENERATE_INTERRUPT_ON_RX |
										  MPSC_BISYNC_IGNORE_CHAR_IN_TEXT_MODE |
										  MPSC_NACK_ODD));
	}

	/* Parity mode high */
	if(PARITY_HIGH == parity)
	{
    	GT_MPSC_CHR2_CONFIG(port, MPSC_BISYNC_RX_ENABLE_VERTICAL_PARITY_CHECK |
    						      MPSC_BISYNC_TX_ENABLE_VERTICAL_PARITY_CHECK |
    						      MPSC_BISYNC_RX_PARITY_HIGH |
    						      MPSC_BISYNC_TX_PARITY_HIGH);
    	

    	GT_MPSC_CHR1_CONFIG(port, 
               MPSC_BISYNC_DISCARD_SYNC |
    	  	   MPSC_BISYNC_DISCARD_DLE |
    	  	   GT_MPSC_BISYNC_SYNC((MPSC_BISYNC_DEFAULT_SYNC_CHAR | 0x80)) |
    	  	   GT_MPSC_BISYNC_DLE((MPSC_BISYNC_DEFAULT_DELIMETER_CHAR | 0x80)));
    		


    	GT_MPSC_CHR6_CONFIG(port, 
            GT_MPSC_BISYNC_CONTROL_CHAR_LOW(MPSC_BISYNC_ENTRY_VALID |
                                          MPSC_BISYNC_START_OF_TEXT |
                                          MPSC_BISYNC_GENERATE_INTERRUPT_ON_RX |
    									  (MPSC_START_OF_TXT | 0x80)) |
    		GT_MPSC_BISYNC_CONTROL_CHAR_HIGH(MPSC_BISYNC_ENTRY_VALID |
                                          MPSC_BISYNC_START_OF_HEADER |
                                          MPSC_BISYNC_GENERATE_INTERRUPT_ON_RX |
    									  (MPSC_START_OF_HNDLR | 0x80)));


    	GT_MPSC_CHR7_CONFIG(port, 
            GT_MPSC_BISYNC_CONTROL_CHAR_LOW(MPSC_BISYNC_ENTRY_VALID |
    		  							  MPSC_BISYNC_MOVE_TO_HUNT |
                                          MPSC_BISYNC_GENERATE_INTERRUPT_ON_RX |
    									  MPSC_BISYNC_BCC_NEXT |
    									  (MPSC_END_OF_TXT | 0x80))|
    		GT_MPSC_BISYNC_CONTROL_CHAR_HIGH(MPSC_BISYNC_ENTRY_VALID |
										 MPSC_BISYNC_MOVE_TO_HUNT |
                                         MPSC_BISYNC_GENERATE_INTERRUPT_ON_RX |
    									 MPSC_BISYNC_BCC_NEXT |
										 (MPSC_END_INTERMEDIATE_BLOCK | 0x80)));

		GT_MPSC_CHR8_CONFIG(port, 
			GT_MPSC_BISYNC_CONTROL_CHAR_LOW(MPSC_BISYNC_ENTRY_VALID |             
									      MPSC_BISYNC_MOVE_TO_HUNT |            
										  MPSC_BISYNC_GENERATE_INTERRUPT_ON_RX |
										  (MPSC_ENQUIRY | 0x80))|                     
			GT_MPSC_BISYNC_CONTROL_CHAR_HIGH(MPSC_BISYNC_ENTRY_VALID |             
										  MPSC_BISYNC_MOVE_TO_HUNT |            
										  MPSC_BISYNC_GENERATE_INTERRUPT_ON_RX |
										  MPSC_BISYNC_IGNORE_CHAR_IN_TEXT_MODE |
										  MPSC_NACK | 0x80));
	}

	/* Parity mode low */
	if(PARITY_LOW == parity)
	{
    	GT_MPSC_CHR2_CONFIG(port, MPSC_BISYNC_RX_ENABLE_VERTICAL_PARITY_CHECK |
        						  MPSC_BISYNC_TX_ENABLE_VERTICAL_PARITY_CHECK |
        						  MPSC_BISYNC_RX_PARITY_LOW |
        						  MPSC_BISYNC_TX_PARITY_LOW);
    	

    	GT_MPSC_CHR1_CONFIG(port, 
               MPSC_BISYNC_DISCARD_SYNC |
    	  	   MPSC_BISYNC_DISCARD_DLE |
    	  	   GT_MPSC_BISYNC_SYNC((MPSC_BISYNC_DEFAULT_SYNC_CHAR & 0x7f)) |
    	  	   GT_MPSC_BISYNC_DLE((MPSC_BISYNC_DEFAULT_DELIMETER_CHAR & 0x7f)));
    		


    	GT_MPSC_CHR6_CONFIG(port, 
            GT_MPSC_BISYNC_CONTROL_CHAR_LOW(MPSC_BISYNC_ENTRY_VALID |
                                          MPSC_BISYNC_START_OF_TEXT |
                                          MPSC_BISYNC_GENERATE_INTERRUPT_ON_RX |
    									  (MPSC_START_OF_TXT & 0x7f)) |
    		GT_MPSC_BISYNC_CONTROL_CHAR_HIGH(MPSC_BISYNC_ENTRY_VALID |
                                          MPSC_BISYNC_START_OF_HEADER |
                                          MPSC_BISYNC_GENERATE_INTERRUPT_ON_RX |
    									  (MPSC_START_OF_HNDLR & 0x7f)));


    	GT_MPSC_CHR7_CONFIG(port, 
            GT_MPSC_BISYNC_CONTROL_CHAR_LOW(MPSC_BISYNC_ENTRY_VALID |
    		  							  MPSC_BISYNC_MOVE_TO_HUNT |
                                          MPSC_BISYNC_GENERATE_INTERRUPT_ON_RX |
    									  MPSC_BISYNC_BCC_NEXT |
    									  (MPSC_END_OF_TXT & 0x7f))|
    		GT_MPSC_BISYNC_CONTROL_CHAR_HIGH(MPSC_BISYNC_ENTRY_VALID |
										 MPSC_BISYNC_MOVE_TO_HUNT |
                                         MPSC_BISYNC_GENERATE_INTERRUPT_ON_RX |
    									 MPSC_BISYNC_BCC_NEXT |
										 (MPSC_END_INTERMEDIATE_BLOCK & 0x7f)));


		GT_MPSC_CHR8_CONFIG(port, 
			GT_MPSC_BISYNC_CONTROL_CHAR_LOW(MPSC_BISYNC_ENTRY_VALID |             
										  MPSC_BISYNC_MOVE_TO_HUNT |            
										  MPSC_BISYNC_GENERATE_INTERRUPT_ON_RX |
										  (MPSC_ENQUIRY & 0x7f))|                     
			GT_MPSC_BISYNC_CONTROL_CHAR_HIGH(MPSC_BISYNC_ENTRY_VALID |             
										  MPSC_BISYNC_MOVE_TO_HUNT |            
										  MPSC_BISYNC_GENERATE_INTERRUPT_ON_RX |
										  MPSC_BISYNC_IGNORE_CHAR_IN_TEXT_MODE |
										  (MPSC_NACK & 0x7f)));
	}
}

/*******************************************************************************
* gtMpscConfigForTRANSPARENT - configures the MPSC for Transparent with default 
*                              values 
*
* DESCRIPTION:
*         Initializes the port to work in default configurations, and in rate 
*         determined by user.
*         Default configurations are: 
*         CRC mode - CRC-16, Transmit and receive encoder - NRZ, DPLL uses both
*         rising and falling edges for synchronization, receive sync length = 
*         16bit, channel works in Transparent synchroniztin mode with sync char 
*         = 0x7e.
*      
* INPUT:
*         port - the port to be initiated.
*         rxClkSrc - Source of rx Clk. put NO_SRC if port is not functioning as 
*                    rx!!!             
*         txClkSrc - Source of tx Clk. put NO_SRC if port is not functioning as 
*                    tx!!! 
*         rxClkRate - desired clock rate of rx. valid only when ClkSrc is one
*                     of the brg's. Put 0 at any other case. 
*         txClkRate - desired clock rate of tx. valid only when ClkSrc is one
*                     of the brg's. Put 0 at any other case.  
*         tclkFreq - in case rxClkSrc or TxClkSrc are BRG0 or BRG1 send T-clock
*                    frequency in this variable (T-clk will be used as the BRG 
*                    source). If another source for the brg is desired use 
*                    gtMpscBrgInit() to configure the BRG.
*                     
*
* OUTPUT:
*       An MPSC_ACTUAL_RATES sturcture that indicates with fields:
*       rxClkActualRate - when user uses brg as clock src the function derives 
*       the actual rate from T-CLk. the actual rate is not always equal to the 
*       desired one (see gtMpscBrgInit()).
*       txClkActualRate - like rx clk.
*
* RETURN:
*       None.
*
*****************************************************************************/
void gtMpscConfigForTRANSPARENT(MPSC_PORT port, MPSC_CLOCK_SOURCE rxClkSrc, 
                                unsigned int rxClkRate, 
                                MPSC_CLOCK_SOURCE txClkSrc,
                                unsigned int txClkRate, unsigned int tclkFreq,
                                MPSC_ACTUAL_RATES  *rates)
{
    GT_MPSC_MRR_CONFIG(port); 
    GT_MPSC_CLOCK_ROUTING_CONFIG(port, rxClkSrc, txClkSrc);
    if(rxClkSrc == BRG0 || rxClkSrc == BRG1)
    {
        gtMpscBrgEnable(rxClkSrc);
        rates->rxClkActualRate = 
            gtMpscBrgInit(rxClkSrc, GT_TCLK, tclkFreq, rxClkRate);
    }
    if(txClkSrc == BRG0 || txClkSrc == BRG1)
    {
        gtMpscBrgEnable(txClkSrc);
        rates->txClkActualRate = 
            gtMpscBrgInit(txClkSrc, GT_TCLK, tclkFreq, txClkRate);
    }   
	if(rxClkSrc != NO_SRC)  /* port functions as rx */
    {
        GT_MPSC_CHR2_CONFIG(port, MPSC_TRANSPARENT_ENTER_HUNT);
    }  
    GT_MPSC_MMCRL_CONFIG(port, MPSC_MODE_TRANSPARENT_TRANSMITTER |
						       MPSC_MODE_TRANSPARENT_RECEIVER |
							   MPSC_CRC16 |
							   MPSC_ENABLE_TRANSMIT |
							   MPSC_ENABLE_RECEIVE |
                               MPSC_CTS_PULSE_MODE);

	GT_MPSC_MMCRH_CONFIG(port, MPSC_TRANSMIT_ENCODER_NRZ |
							   MPSC_RECEIVE_ENCODER_NRZ |
		 					   MPSC_SYNCHRONIZE_ON_FALLING_AND_RISING |
	 						   MPSC_RECEIVE_SYNC_LENGTH_16_BIT);

	GT_MPSC_CHR1_CONFIG(port, MPSC_TRANSPARENT_SYNC_MODE |
                   GT_MPSC_TRANSPARENT_SYNC_CHAR(MPSC_TRANS_DEFAULT_SYNC_CHAR));

}

/*******************************************************************************
* gtMpscConfigForHDLC - configs the MPSC for HDLC with default values 
*
* DESCRIPTION:
*      Initializes the port to work in default configurations, and in rate 
*      determined by user.
*      Default configurations are:                                              
*      Null Modem, Synchronized with sync char 0x7e, No. of flags between 
*      consecutive frames = 1, max frame length 64k (0xffff), No address  
*      filtering, Broadcast and null addresses packets reception is       
*      enabled. 
* 
* INPUT:
*      port - the port to be initiated.
*      rxClkSrc - Source of rx Clk. put NO_SRC if port is not functioning as 
*                 rx!!!             
*      txClkSrc - Source of tx Clk. put NO_SRC if port is not functioning as 
*                 tx!!! 
*      rxClkRate - desired clock rate of rx. valid only when ClkSrc is one
*                  of the brg's. Put 0 at any other case. 
*      txClkRate - desired clock rate of tx. valid only when ClkSrc is one
*                  of the brg's. Put 0 at any other case.  
*      tclkFreq - in case rxClkSrc or TxClkSrc are BRG0 or BRG1 send T-clock
*                 frequency in this variable (T-clk will be used as the BRG 
*                 source). If another source for the brg is desired use 
*                 gtMpscBrgInit() to configure the BRG.
*
* OUTPUT:
*      An MPSC_ACTUAL_RATES sturcture that indicates with fields:
*      rxClkActualRate - when user uses brg as clock src the function derives 
*      the actual rate from T-CLk. the actual rate is not always equal to the 
*      desired one (see gtMpscBrgInit()).
*      txClkActualRate - like rx clk.
*
* RETURN:
*      None.
*
*******************************************************************************/
void gtMpscConfigForHDLC(MPSC_PORT port, MPSC_CLOCK_SOURCE rxClkSrc, 
                         unsigned int rxClkRate, MPSC_CLOCK_SOURCE txClkSrc,
                         unsigned int txClkRate, unsigned int tclkFreq,
                         MPSC_ACTUAL_RATES *rates)
{
    GT_MPSC_MRR_CONFIG(port); 
    GT_MPSC_CLOCK_ROUTING_CONFIG(port, rxClkSrc, txClkSrc);
    if(rxClkSrc == BRG0 || rxClkSrc == BRG1)
    {
        gtMpscBrgEnable(rxClkSrc);
        rates->rxClkActualRate = 
            gtMpscBrgInit(rxClkSrc, GT_TCLK, tclkFreq, rxClkRate);  
    }

    if(txClkSrc == BRG0 || txClkSrc == BRG1)
    {
        gtMpscBrgEnable(txClkSrc);
        rates->txClkActualRate = 
            gtMpscBrgInit(txClkSrc, GT_TCLK, tclkFreq, txClkRate); 

    }   
	if(rxClkSrc != NO_SRC)  /* port functions as rx */
    {
        GT_MPSC_CHR2_CONFIG(port, MPSC_HDLC_ENTER_HUNT);
    }      

    GT_MPSC_MMCRL_CONFIG(port, MPSC_MODE_HDLC |
							   MPSC_CRC32_CCITT |
							   MPSC_ENABLE_TRANSMIT |
						       MPSC_ENABLE_RECEIVE |
							   MPSC_NULL_MODEM);

    /* synchroniztion done using sync char (defined in char1) */
	GT_MPSC_MMCRH_CONFIG(port, MPSC_TRANSMIT_ENCODER_NRZ |
						       MPSC_RECEIVE_ENCODER_NRZ |
		 					   MPSC_SYNCHRONIZE_ON_FALLING_AND_RISING |
	 						   MPSC_RECEIVE_SYNC_LENGTH_16_BIT);

	GT_MPSC_MPCR_CONFIG(port, GT_MPSC_HDLC_NUMBER_OF_FLAGS(1));
	
	GT_MPSC_CHR1_CONFIG(port, GT_MPSC_HDLC_SYNC(MPSC_HDLC_DEFAULT_SYNC_CHAR) |
                              GT_MPSC_HDLC_ABORT(MPSC_HDLC_DEFAULT_ABORT_CHAR));

	GT_MPSC_CHR3_CONFIG(port, GT_MPSC_HDLC_FRAME_LENGTH(0xffff));

	GT_MPSC_CHR4_CONFIG(port, MPSC_HDLC_NULL_ENABLE |
						      MPSC_HDLC_BROADCAST_ENABLE |
						      GT_MPSC_HDLC_BCE_BITS(MPSC_DEFAULT_NULL_VALUE));
}


/*******************************************************************************
* gtMpscConfigForUART - configures the MPSC for UART with default values 
*
* DESCRIPTION:
*      Initializes the port to work in default configurations, and in rate 
*      determined by user.
*      Default configurations are: 
*      data reverse mode for rx and for tx, CRC mode - CRC-16, Transmit and 
*      receive encoder - NRZ, DPLL uses both rising and falling edges for 
*      synchronization, receive sync length =  16bit, asynchronous mode, char 
*      length 8 bit, 1 stop bit, break count = 0, control stuff char = 0, 
*      receive and transmit parity mode = even, max idle = 0, control filtering
*      register = 0.  
* 
* INPUT:
*      port - the port to be initiated.
*      rxClkSrc - Source of rx Clk. put NO_SRC if port is not functioning as 
*                 rx!!!             
*      txClkSrc - Source of tx Clk. put NO_SRC if port is not functioning as 
*                 tx!!! 
*      rxClkRate - desired clock rate of rx. valid only when ClkSrc is one
*                  of the brg's. Put 0 at any other case. 
*      txClkRate - desired clock rate of tx. valid only when ClkSrc is one
*                  of the brg's. Put 0 at any other case.  
*      tclkFreq - in case rxClkSrc or TxClkSrc are BRG0 or BRG1 send T-clock
*                 frequency in this variable (T-clk will be used as the BRG 
*                 source). If another source for the brg is desired use 
*                 gtMpscBrgInit() to configure the BRG.
*
* OUTPUT:
*      An MPSC_ACTUAL_RATES sturcture that indicates with fields:
*      rxClkActualRate - when user uses brg as clock src the function derives 
*      the actual rate from T-CLk. the actual rate is not always equal to the 
*      desired one (see gtMpscBrgInit()).
*      txClkActualRate - like rx clk.
*
* RETURN:
*       None.
*
*******************************************************************************/
void gtMpscConfigForUART(MPSC_PORT port, MPSC_CLOCK_SOURCE rxClkSrc, 
                         unsigned int rxClkRate, MPSC_CLOCK_SOURCE txClkSrc,
                         unsigned int txClkRate, unsigned int tclkFreq,
                         MPSC_ACTUAL_RATES *rates)
{
    GT_MPSC_MRR_CONFIG(port); 
    GT_MPSC_CLOCK_ROUTING_CONFIG(port, rxClkSrc, txClkSrc);
    if((rxClkSrc == BRG0 || rxClkSrc == BRG1) && (rates != NULL))
    {
        rates->rxClkActualRate = 
            gtMpscBrgInit(rxClkSrc, GT_TCLK, tclkFreq, rxClkRate);
    }
    if(txClkSrc == BRG0 || txClkSrc == BRG1)
    {
        rates->txClkActualRate = 
            gtMpscBrgInit(txClkSrc, GT_TCLK, tclkFreq, txClkRate);

    }   
	if(rxClkSrc != NO_SRC)  /* port functions as rx */
    {
        GT_MPSC_CHR2_CONFIG(port, MPSC_UART_ENTER_HUNT);
          
    }
    
    GT_MPSC_MMCRL_CONFIG(port, MPSC_MODE_UART |
	         				   MPSC_CRC16 |
							   MPSC_ENABLE_TRANSMIT |
							   MPSC_ENABLE_RECEIVE |
                               MPSC_NULL_MODEM);  



   
    GT_MPSC_MMCRH_CONFIG(port, MPSC_TRANSMIT_ENCODER_NRZ |
							   MPSC_RECEIVE_ENCODER_NRZ |
		 					   MPSC_SYNCHRONIZE_ON_FALLING_AND_RISING |
                               MPSC_RECEIVE_CLOCK_DEVIDER_32x |
                               MPSC_TRANSMIT_CLOCK_DEVIDER_32x |
                               MPSC_RECEIVE_DATA_WIDTH_8BIT);  

     GT_MPSC_MPCR_CONFIG(port, MPSC_UART_NORMAL_MODE |
						       MPSC_UART_ASYNCHRONOUS_MODE |
						       MPSC_UART_CHAR_LENGTH_8_BIT | 
						       MPSC_UART_STOP_BIT_LENGTH_2 | 
						       MPSC_UART_FLOW_CONTROL_NORMAL);
    
    GT_MPSC_CHR1_CONFIG(port,
                        GT_MPSC_UART_CONTROL_STUFF_CHAR(MPSC_DEFAULT_NULL_VALUE) 
                        | GT_MPSC_UART_BREAK_COUNT(MPSC_DEFAULT_NULL_VALUE)); 

		
    GT_MPSC_CHR2_CONFIG(port, MPSC_UART_ENTER_HUNT);

	GT_MPSC_CHR3_CONFIG(port, GT_MPSC_UART_MAX_IDLE(MPSC_DEFAULT_NULL_VALUE));

    GT_MPSC_CHR4_CONFIG(port, 
                       GT_MPSC_UART_CONTROL_FILTERING(MPSC_DEFAULT_NULL_VALUE));
    
}

/*******************************************************************************
* gtMpscBrgEnable - enables given brg engine
*
* DESCRIPTION:
*      Set enable bit in the brg's configuration register.
*
* INPUT:
*      brgNum
*
* OUTPUT:
*      None.
*
* RETURN:
*      None.
*
*******************************************************************************/
void gtMpscBrgEnable(MPSC_BRG_NUM brgNum)
{
    GT_SET_REG_BITS(BRG_CONFIG_REG(brgNum), MPSC_ENABLE_BRG);
}

/*******************************************************************************
* gtMpscBrgDisable - disables given brg engine
*
* DESCRIPTION:
*      Reset enable bit in the brg's configuration register.
*
* INPUT:
*      brgNum
*
* OUTPUT:
*      None.
*
* RETURN:
*      None.
*
*******************************************************************************/
void gtMpscBrgDisable(MPSC_BRG_NUM brgNum)
{
    GT_RESET_REG_BITS(BRG_CONFIG_REG(brgNum), MPSC_ENABLE_BRG);
}
/*******************************************************************************
* gtMpscBrgInit - initializes brg engine according to given parameters
*
* DESCRIPTION:
*       The function initializes a given brg engine, using the brg source given 
*       by  the user.
*
* INPUT:
*       brgNum - the brg to be initiated
*       brgClockSource - the source clock for the brg, defined in gtMpsc.h
*       brgClockSourceFreq - the source clock frequency.
*       freq - the requested frequency (the actual frequency can be different).
*
* OUTPUT:
*       None.
*
* RETURN:
*       Actual clock frequency (in Hz) produced by brg. The actual frequency can
*       be different from requested freq because brg can produce frequencies 
*       that equal to source clock frequency or to source clock freq/(2*n), 
*       for 1<n<65536. The brg actual freq will be the closest possible to the 
*       requested one. 
*
*******************************************************************************/
unsigned int gtMpscBrgInit(MPSC_BRG_NUM brgNum, 
                           MPSC_BRG_CLOCK_SOURCE brgClockSource,
                           unsigned int brgClockSourceFreq, unsigned int freq)
{
    double       doubleCdv;
    unsigned int CDV, valueForBrgRegister;

    if(freq == 0)
    {
        /* Disable the BRG */
        GT_RESET_REG_BITS(BRG_CONFIG_REG(brgNum), MPSC_ENABLE_BRG);
        return 0;
    }

    GT_REG_READ(BRG_CONFIG_REG(brgNum), &valueForBrgRegister);
   

    valueForBrgRegister = 
         (valueForBrgRegister & 0xff83ffff) | (brgClockSource << 18);

           
    if(brgClockSourceFreq == freq)
    {
        CDV = 0;
    }
    else
    {
        doubleCdv = ((double)brgClockSourceFreq / (double)(freq*2)) - 1.0;
        if(doubleCdv < 1)
        {
            CDV = 1;
        }
        else if(doubleCdv > 0xffff)
        {
            CDV = 0xffff;
        }
        else
        {
            CDV = (unsigned int)doubleCdv;
            /* CDV is rounded up to the closest integer: */
            if((doubleCdv - (double)CDV) >= 0.5) CDV += 1; 
        }
    }
    
    valueForBrgRegister = valueForBrgRegister | (CDV & MPSC_CDV_MASK);

    GT_REG_WRITE(BRG_CONFIG_REG(brgNum), valueForBrgRegister | MPSC_ENABLE_BRG);
    if(CDV == 0)
    {
        return freq;
    }
    return (brgClockSourceFreq / ((CDV+1) * 2));
}


/*******************************************************************************
* gtAbortMpsc - abort rx & tx at mpsc port
*
* DESCRIPTION:
*		Abort rx & tx at mpsc port. For aborting only tx or rx use macros 
*       GT_MPSC_ABORT_RX and GT_MPSC_ABORT_TX.
*
* INPUT:
*	  port - mpsc port to be aborted.
*
* OUTPUT:
*		None.
*
* RETURN:
*		None.
*
*******************************************************************************/
void gtAbortMpsc(MPSC_PORT port)
{
   /* good for all protocols */
   GT_SET_REG_BITS(MPSC_CHANNEL_REG_2(port), 
                    MPSC_HDLC_ABORT_RX | MPSC_HDLC_ABORT_TX); 
}

/*******************************************************************************
* gtAbortAll - abort all port sdma & mpsc
*
* DESCRIPTION:
*		Abort all port sdma & mpsc.
*
* INPUT:
*		numOfPorts - num of ports (currently 2 for discoA).
*
* OUTPUT:
*		None.
*
* RETURN:
*		None.
*
*******************************************************************************/
void gtAbortAllMpscPorts()
{
    int numOfPorts, numOfGroups, i, j;
#ifdef INCLUDE_MPSC_PORTS_2_7
    numOfPorts = 8;
#else
    numOfPorts = 2;
#endif /* INCLUDE_MPSC_PORTS_2_7 */

#ifdef INCLUDE_SDMA_GROUP_1
    numOfGroups = 2;
#else 
    numOfGroups = 1;
#endif /* INCLUDE_SDMA_GROUP_1 */
    for(i = 0; i < numOfPorts; i++)
    {
        for (j = 0; j < numOfGroups; j++)
        {
            gtAbortSdma(j, i);
        }
        gtAbortMpsc(i);
    }
}

/*******************************************************************************
* areAllMpscPortsAborted - check if all ports are aborted 
*
* DESCRIPTION:
*		Checks if all ports are aborted .
*
* INPUT:
*		None.
*
* OUTPUT:
*		None.
*
* RETURN:
*		true - If all ports are aborted.
*       false - If any sdma channel or its matching MPSC port is working.
*
*******************************************************************************/
bool areAllMpscPortsAborted()
{
    int numOfPorts, numOfGroups,i ,j;

#ifdef INCLUDE_MPSC_PORTS_2_7
    numOfPorts = 8;
#else
    numOfPorts = 2;
#endif /* INCLUDE_MPSC_PORTS_2_7 */

#ifdef INCLUDE_SDMA_GROUP_1
    numOfGroups = 2;
#else 
    numOfGroups = 1;
#endif /* INCLUDE_SDMA_GROUP_1 */
        
    for (i = 0; i < numOfPorts; i++)
    {
        for (j = 0; j < numOfGroups; j++)
        {
            if ((GTREGREAD(SDMA_COMM_REG(j, i))) & 
                          (SDMA_ABORT_TX_BIT | SDMA_ABORT_RX_BIT)) return false;
        }
        if((GTREGREAD(MPSC_CHANNEL_REG_2(i))) & (BIT7 | BIT23)) return false;                     
    }
    return true;    
}












